;nyquist plug-in
;version 1
;type process
;name "Nonlinear Maximizer Tanh"
;action "Nonlinear Maximizer Tanh"
;info "by Igor Chernenko\nReleased under terms of GNU Public License.\nThis is, perhaps, the most advanced version of Nonlinear Maximizer. It works wonders.\nThe purpose of Nonlinear Maximizer is to increase loudness. It can perform gentle\ncompression of classic music as well as strong compression of speech recordings.\nHow to process iRiver E100 recordings:\n1. drug-and-drop your wma-file into Audacity (v.1.3.6);\n2. switch to 32-bit resolution;\n3. remove DC offset (with Normalizer);\n4. apply Band Pass Filter;\n5. apply Low Pass Filter (8kHz, 6dB);\n6. apply Noise Removal with default settings (3 times);\n7. apply Band Pass Filter;\n8. normalize wave to 0dB;\n9. apply Band Pass Filter;\n10. apply the Maximizer;\n11. apply Band Pass Filter.\n12. apply Noise Removal with default settings;\n13. apply Band Pass Filter;\n14. save the result in FLAC format."


;control power-comp "maximize" real "" 1 0.1 20

(if (< power-comp 0.1) (setq power-comp 0.1))

(display "maximize" power-comp)

; y = tanh(k*x)/tanh(k)
; k = power-comp

; Nonlinear Maximizer Tanh by Igor Chernenko
; updated December 1, 2008
; Released under terms of the GNU Public License
; http://www.opensource.org/licenses/gpl-license.php


; HOW IT WORKS
;
; Nonlinear Maximizer Tanh performs smooth non-linear transformation of the sound rather 
; than classic compression. Afterwards, the sound is amplified to the previous level. 
;
; The magic formula of compression is hyperbolic tangent:
;
; y = tanh(k*x)/tanh(k)
;
; x = sound
;
; y = compressed sound
;
; This is, perhaps, the most advanced version of Nonlinear Maximizer. It works wonders.
;
; Wikipedia: Hyperbolic tangent, often pronounced "tanch" (or "than", with the "th" like in "Thursday")
; http://en.wikipedia.org/wiki/Hyperbolic_function
;
; Edgar-RFT: Logarithmic math with Nyquist and XLISP
; http://n2.nabble.com/plugin:-distortion-via-nth-root-function-td238782.html




; (mult a [b c ...])   Returns the product of a, b, c, ..., allowing mixed multiplication 
;                      of sounds, multichannel sounds and numbers.

; n-th power of x:     (exp (* (log x) n))
; x^(-1.0) = 1/x:      (exp (* (log x) -1.0))    [ x > 0 ]


(defun s-hyperbolic (mysound) 
  (s-exp 
  (mult (s-log mysound) -1.0)
  )
)

; tanh(x) = (exp(x) - exp(-x))/(exp(x) + exp(-x))


(defun tanh (r-number)
  (/     
  (- (exp r-number) (exp (* r-number -1.0)))
  (+ (exp r-number) (exp (* r-number -1.0)))
  )
)

(setq max-amplitude (+ (tanh power-comp) 0.000001))

(setq amplify-it (/ 1 max-amplitude))

(display "max-amplitude" max-amplitude)

(display "amplify-it" amplify-it)


(defun s-tanh (mysound)
 (mult
    (sum (s-exp mysound) (mult (s-exp (mult mysound -1.0)) -1.0) )  
    (s-hyperbolic
      (sum (s-exp mysound) (s-exp (mult mysound -1.0)) )  
    )
 )
)


(if (arrayp s) 
    (scale amplify-it
      (vector 
          (s-tanh (mult (aref s 0) power-comp))
          (s-tanh (mult (aref s 1) power-comp))
      )
    )
    (scale amplify-it
    (s-tanh (mult s power-comp))
    )
)

